--[[
# Copyright 2001-2014 Cisco Systems, Inc. and/or its affiliates. All rights
# reserved.
#
# This file contains proprietary Detector Content created by Cisco Systems,
# Inc. or its affiliates ("Cisco") and is distributed under the GNU General
# Public License, v2 (the "GPL").  This file may also include Detector Content
# contributed by third parties. Third party contributors are identified in the
# "authors" file.  The Detector Content created by Cisco is owned by, and
# remains the property of, Cisco.  Detector Content from third party
# contributors is owned by, and remains the property of, such third parties and
# is distributed under the GPL.  The term "Detector Content" means specifically
# formulated patterns and logic to identify applications based on network
# traffic characteristics, comprised of instructions in source code or object
# code form (including the structure, sequence, organization, and syntax
# thereof), and all documentation related thereto that have been officially
# approved by Cisco.  Modifications are considered part of the Detector
# Content.
--]]
--[[
detection_name: SCCP
version: 3
description: Skinny Call Control Protocol, a VoIP call setup protocol.
--]]

require "DetectorCommon"

--require('debugger')

--local DC = require("DetectorCommon")
local DC = DetectorCommon
local HT = hostServiceTrackerModule
local FT = flowTrackerModule

DetectorPackageInfo = {
    name =  "SCCP",
    proto =  DC.ipproto.tcp,
    server = {
        init =  'DetectorInit',
        validate =  'DetectorValidator',
    }
}

gServiceId = 20196
gServiceName = 'SCCP'
gSfAppIdSCCP = 2940
gSfAppIdRTP = 813

gPatterns = {
    register_msg = {'\001\000\000\000', 8, gSfAppIdSCCP},
    register_ack = {'\129\000\000\000', 8, gSfAppIdSCCP},
    register_reject = {'\157\000\000\000', 8, gSfAppIdSCCP},
    unregister_msg = {'\039\000\000\000', 8, gSfAppIdSCCP},
    unregister_ack = {'\024\001\000\000', 8, gSfAppIdSCCP},
    startmedia_msg = {'\138\000\000\000', 8, gSfAppIdSCCP},
    alarm_msg = {'\032\000\000\000', 8, gSfAppIdSCCP},
    keepalive_msg = {'\000\000\000\000', 8, gSfAppIdSCCP},
    keepalive_ack = {'\000\001\000\000', 8, gSfAppIdSCCP},    
    offhook_msg = {'\006\000\000\000', 8, gSfAppIdSCCP},
    setringer_msg = {'', 8, gSfAppIdSCCP},
}

gFastPatterns = {
    {DC.ipproto.tcp, gPatterns.register_msg},
    {DC.ipproto.tcp, gPatterns.register_ack},
}

gPorts = {
    {DC.ipproto.tcp, 2000},
    {DC.ipproto.udp, 2000},
}

gAppRegistry = {
	{gSfAppIdSCCP, 0}
}

function serviceInProcess(context)

    local flowFlag = context.detectorFlow:getFlowFlag(DC.flowFlags.serviceDetected)

    if ((not flowFlag) or (flowFlag == 0)) then
        gDetector:inProcessService()
    end

    DC.printf('%s: Inprocess, packetCount: %d\n', gServiceName, context.packetCount);
    return DC.serviceStatus.inProcess
end

function serviceSuccess(context)
    local flowFlag = context.detectorFlow:getFlowFlag(DC.flowFlags.serviceDetected)

    if ((not flowFlag) or (flowFlag == 0)) then
        gDetector:addService(gServiceId, "", "", gSfAppIdSCCP)
    end

    DC.printf('%s: Detected, packetCount: %d\n', gServiceName, context.packetCount);
    return DC.serviceStatus.success
end

function serviceFail(context)
    local flowFlag = context.detectorFlow:getFlowFlag(DC.flowFlags.serviceDetected)

    if ((not flowFlag) or (flowFlag == 0)) then
        gDetector:failService()
    end

    context.detectorFlow:clearFlowFlag(DC.flowFlags.continue)
    DC.printf('%s: Failed, packetCount: %d\n', gServiceName, context.packetCount);
    return DC.serviceStatus.nomatch
end

function registerPortsPatterns()

    for i,v in ipairs(gFastPatterns) do
        if ( gDetector:registerPattern(v[1], v[2][1], #v[2][1], v[2][2], v[2][3]) ~= 0) then
            DC.printf ('%s: register pattern failed for %s\n', gServiceName,v[2][1])
        else
            DC.printf ('%s: register pattern successful for %s\n', gServiceName,v[2][1])
        end
    end

    for i,v in ipairs(gPorts) do
        gDetector:addPort(v[1], v[2])
    end


	for i,v in ipairs(gAppRegistry) do
		pcall(function () gDetector:registerAppId(v[1],v[2]) end)
	end

end

function DetectorInit( detectorInstance)

    gDetector = detectorInstance
    DC.printf('%s: DetectorInit()\n',gServiceName)

    gDetector:init(gServiceName, 'DetectorValidator', 'DetectorFini')
    registerPortsPatterns()

    if (gDetector.AFAddApp) then
        gDetector:AFAddApp(2940, 813, 2940);
    end

    return gDetector
end

function DetectorValidator()
    local context = {}
    context.detectorFlow = gDetector:getFlow()
    context.packetDataLen = gDetector:getPacketSize()
    context.packetDir = gDetector:getPacketDir()
    context.srcIp = gDetector:getPktSrcAddr()
    context.dstIp = gDetector:getPktDstAddr()
    context.srcPort = gDetector:getPktSrcPort()
    context.dstPort = gDetector:getPktDstPort()
    context.flowKey = context.detectorFlow:getFlowKey()
    context.packetCount = gDetector:getPktCount()
    local size = context.packetDataLen
    local dir = context.packetDir
    local srcPort = context.srcPort
    local dstPort = context.dstPort
    local flowKey = context.flowKey

    DC.printf ('%s:DetectorValidator(): packetCount %d, dir %d, size %d\n', gServiceName, context.packetCount, dir, size);

    if (size == 0 or dir == 0) then
        return serviceInProcess(context)
    end

    local rft = FT.getFlowTracker(flowKey)
    if (not rft) then
            rft = FT.addFlowTracker(flowKey, {seen=0})
    end

    if (dir == 1 and size > 11) then  
        matched, len_raw = gDetector:getPcreGroups("(....)", 0)
        if (matched) then
            len = DC.reverseBinaryStringToNumber(len_raw, 4)
            DC.printf ('%s:DetectorValidator(): len is %d size is %d\n', gServiceName, len, size)
            -- some unknown message 
            if (size - 8 == len or gDetector:memcmp("\069\001\000\000", 4, 8) == 0) then
                DC.printf ('%s:DetectorValidator(): size-8=len, good...\n', gServiceName)
                rft.seen = 1
                -- at index 8 we have the 4-byte "message id" field
                -- we are interested in StartMediaTransfer messages, 0x0000008a
                if ((gDetector:memcmp("\138\000\000\000", 4, 8) == 0) and size > 27) then
                    DC.printf ('%s:DetectorValidator(): StartMediaTransfer found\n', gServiceName)
                    matched, dst_ip_raw, dst_port_raw = gDetector:getPcreGroups("(....)(....)", 20)
                    if (matched and gDetector.createFutureFlow) then
                        dst_ip_int = DC.reverseBinaryStringToNumber(dst_ip_raw, 4)
                        dst_port = DC.reverseBinaryStringToNumber(dst_port_raw, 4)
                        -- the PACKET dst IP is going to be the client here, and therefore the src addr of the new flow
                        src_ip_str = intToIPv4 (context.dstIp, 1)
                        dst_ip_str = intToIPv4 (dst_ip_int, 1)
                        DC.printf ('%s:DetectorValidator(): creating future flow %s:%d - %s:%d\n', gServiceName, src_ip_str, 0, dst_ip_str, dst_port)
                        gDetector:createFutureFlow(src_ip_str, 0, dst_ip_str, dst_port, 17, gSfAppIdRTP, gSfAppIdRTP, gSfAppIdSCCP, gSfAppIdSCCP)
                        gDetector:createFutureFlow(dst_ip_str, dst_port, src_ip_str, 0, 17, gSfAppIdRTP, gSfAppIdRTP, gSfAppIdSCCP, gSfAppIdSCCP)
                        return serviceSuccess(context)
                    end
                end

                -- We don't want to stay in process forever waiting for a call
                -- But 150 packets is just a guess - this number may need to be higher
                -- raise it if we get cases where we aren't IDing the RTP traffic
                if (context.packetCount > 150) then
                    return serviceSuccess(context)
                else
                    return serviceInProcess(context)
                end
            end
        end
    end

    if (rft.seen == 1) then
        return serviceSuccess(context)
    else
        return serviceFail(context)
    end

end

--[[Required DetectorFini function
--]]
function DetectorFini()
    --print (gServiceName .. ': DetectorFini()')
end
